#include <SDL/SDL.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>

void Eval(double m,double n1,double n2,double n3,double phi,double &x,double &y)
{
  double r;
  double t1,t2;
  double a=1,b=1;
  
  t1 = cos(m * phi / 4) / a;
  t1 = fabs(t1);
  t1 = pow(t1,n2);
  
  t2 = sin(m * phi/ 4) / b;
  t2 = fabs(t2);
  t2 = pow(t2,n3);
  
  r = pow(t1+t2,-1/n1);

  x = r * cos(phi);
  y = r * sin(phi);
}

int main(int argc, char** argv)
{
    SDL_Surface      *screen, *backup_screen;
    SDL_Event        event;
    SDL_Rect         dst_rect, full_image_rect, scrolled_rect, location;
    int              window_width, window_height, thumb_height, thumb_border_size;

    double           m, n1, n2, n3, phi;

    if(argc < 5) {
      cerr << "usage: " << argv[0] << " m n1 n2 n3" << endl;
      exit(-1);
    } // if

    m = atof(argv[1]);

    n1 = atof(argv[2]);
    n2 = atof(argv[3]);
    n3 = atof(argv[4]);
 
    window_width = 600; window_height = 550;

    // analyze command line

    if(SDL_Init(SDL_INIT_VIDEO) < 0) {
	cerr << "Could not initialize SDL video" << endl;
	exit(-1);
    } // if
    atexit(SDL_Quit);

    Uint32 flags = 0;

    // flags |= SDL_FULLSCREEN;

    screen = SDL_SetVideoMode(window_width, window_height, 24, flags);

    if(!screen) {
	cerr << "could not SetVideoMode() - abort" << endl;
	exit(-1);
    } // if

    long size_line = screen->pitch;

    const long NO_DOTS = 80000;
    const double FACTOR =  1.0;

    double *x, *y;

    x = new double [NO_DOTS];
    y = new double [NO_DOTS];

    double max_x, max_y, min_x, min_y;

    max_x = -1; max_y = -1;
    min_x = 10000; min_y = 10000;

    for(int i = 0; i < NO_DOTS; i++) {
	
	double phi;

	phi = (double)i * 2.0 * M_PI * (double)FACTOR / (double)NO_DOTS; 

	Eval(m, n1, n2, n3, phi, x[i], y[i]);

	if(max_x < x[i]) max_x = x[i];
	if(max_y < y[i]) max_y = y[i];

	if(min_x > x[i]) min_x = x[i];
	if(min_y > y[i]) min_y = y[i];
    } // for

    double factor = max_y - min_y;
    double window_size = window_height;

    if((max_x - min_x) > factor) {
	factor = max_x - min_x;
	window_size = window_height;
    } // if

    for(int i = 0; i < NO_DOTS; i++) {
	
	long ix, iy;

	ix = (long)((x[i]-min_x)*(window_size-40)/factor+20);
	iy = (long)((y[i]-min_y)*(window_size-40)/factor+20);

	if((ix >= 0) && (iy >= 0)) {

	  // draw a while pixel (red == green == blue == 255) here ...
	  
	  ((unsigned char*)(screen->pixels))[ix*3+0 + iy*screen->pitch] = 255;
	  ((unsigned char*)(screen->pixels))[ix*3+1 + iy*screen->pitch] = 255;
	  ((unsigned char*)(screen->pixels))[ix*3+2 + iy*screen->pitch] = 255;
	} // if

    } // for

    delete[] y;
    delete[] x;

    // do not forget to update the pixel-buffer here...

    SDL_UnlockSurface(screen);
    SDL_UpdateRect(screen, 0, 0, window_width, window_height);
    SDL_LockSurface(screen);

    for(;;) {

	if((SDL_PollEvent(&event) == 0) && (event.type == SDL_KEYDOWN)) 
	    goto exit_app;

	switch(event.type) {
		
	    case SDL_MOUSEMOTION:	      
		break;
		
	    case SDL_MOUSEBUTTONDOWN:
		break;
		
	    case SDL_MOUSEBUTTONUP:
		break;
		
	    case SDL_KEYDOWN:
		
		switch(event.key.keysym.sym) {
		    
		    case 27:
		    case 'q':
		    case 'Q':
			goto exit_app;
			
		    default:
			break;
		} // switch
	} // switch

    } // for
	    
 exit_app:

    SDL_UnlockSurface(screen);	    
    SDL_FreeSurface(screen);

    exit(0);
} // main
